@@ -143,17 +143,18 @@ typedef enum
  * pango_default_break:
  * @text: text to break. Must be valid UTF-8
  * @length: length of text in bytes (may be -1 if @text is nul-terminated)
- * @analysis: (nullable): a #PangoAnalysis for the @text
+ * @analysis: (nullable): a `PangoAnalysis` structure for the @text
  * @attrs: logical attributes to fill in
  * @attrs_len: size of the array passed as @attrs
  *
  * This is the default break algorithm.
  *
- * It applies Unicode rules without language-specific tailoring, therefore
- * the @analyis argument is unused and can be %NULL.
+ * It applies Unicode rules without language-specific
+ * tailoring, therefore the @analyis argument is unused
+ * and can be %NULL.
  *
- * See pango_tailor_break() for language-specific breaks.
- **/
+ * See [func@Pango.tailor_break] for language-specific breaks.
+ */
 void
 pango_default_break (const gchar   *text,
 		     gint           length,
@@ -408,6 +409,12 @@ pango_default_break (const gchar   *text,
                 GB_type = GB_Prepend;
                 break;
               }
+            /* Tag chars */
+            if (wc >= 0xE0020 && wc <= 0xE00FF)
+              {
+                GB_type = GB_Extend;
+                break;
+              }
             G_GNUC_FALLTHROUGH;
 	  case G_UNICODE_CONTROL:
 	  case G_UNICODE_LINE_SEPARATOR:
@@ -472,6 +479,7 @@ pango_default_break (const gchar   *text,
             break;
 
           case G_UNICODE_MODIFIER_SYMBOL:
+            /* Fitzpatrick modifiers */
             if (wc >= 0x1F3FB && wc <= 0x1F3FF)
               GB_type = GB_Extend;
             break;
@@ -1617,19 +1625,20 @@ tailor_break (const char    *text,
 
 /**
  * pango_break:
- * @text:      the text to process. Must be valid UTF-8
- * @length:    length of @text in bytes (may be -1 if @text is nul-terminated)
- * @analysis:  #PangoAnalysis structure from pango_itemize()
- * @attrs:     (array length=attrs_len): an array to store character
- *             information in
+ * @text: the text to process. Must be valid UTF-8
+ * @length: length of @text in bytes (may be -1 if @text is nul-terminated)
+ * @analysis: `PangoAnalysis` structure for @text
+ * @attrs: (array length=attrs_len): an array to store character information in
  * @attrs_len: size of the array passed as @attrs
  *
- * Determines possible line, word, and character breaks for a string of
- * Unicode text with a single analysis.
+ * Determines possible line, word, and character breaks
+ * for a string of Unicode text with a single analysis.
  *
- * For most purposes you may want to use pango_get_log_attrs().
+ * For most purposes you may want to use
+ * [func@Pango.get_log_attrs].
  *
- * Deprecated: 1.44: Use pango_default_break() and pango_tailor_break()
+ * Deprecated: 1.44: Use [func@Pango.default_break] and
+ *   [func@Pango.tailor_break]
  */
 void
 pango_break (const gchar   *text,
@@ -1656,15 +1665,18 @@ pango_break (const gchar   *text,
  *
  * Locates a paragraph boundary in @text.
  *
- * A boundary is caused by delimiter characters, such as a newline, carriage
- * return, carriage return-newline pair, or Unicode paragraph separator character.
- * The index of the run of delimiters is returned in @paragraph_delimiter_index.
- * The index of the start of the paragrap (index after all delimiters) is stored
+ * A boundary is caused by delimiter characters, such as
+ * a newline, carriage return, carriage return-newline pair,
+ * or Unicode paragraph separator character.
+ *
+ * The index of the run of delimiters is returned in
+ * @paragraph_delimiter_index. The index of the start
+ * of the paragrap (index after all delimiters) is stored
  * in @next_paragraph_start.
  *
- * If no delimiters are found, both @paragraph_delimiter_index and
- * @next_paragraph_start are filled with the length of @text (an index one
- * off the end).
+ * If no delimiters are found, both @paragraph_delimiter_index
+ * and @next_paragraph_start are filled with the length of @text
+ * (an index one off the end).
  */
 void
 pango_find_paragraph_boundary (const gchar *text,
@@ -1750,16 +1762,18 @@ pango_find_paragraph_boundary (const gchar *text,
  * pango_tailor_break:
  * @text: text to process. Must be valid UTF-8
  * @length: length in bytes of @text
- * @analysis: `PangoAnalysis` structure from [func@itemize] for @text
+ * @analysis: `PangoAnalysis` for @text
  * @offset: Byte offset of @text from the beginning of the
- *     paragraph, or -1 to ignore attributes from @analysis
+ *   paragraph, or -1 to ignore attributes from @analysis
  * @log_attrs: (array length=log_attrs_len): array with one `PangoLogAttr`
  *   per character in @text, plus one extra, to be filled in
  * @log_attrs_len: length of @log_attrs array
  *
- * Apply language-specific tailoring to the breaks in @log_attrs.
+ * Apply language-specific tailoring to the breaks
+ * in @log_attrs.
  *
- * The line breaks are assumed to have been produced by [func@default_break].
+ * The line breaks are assumed to have been produced
+ * by [func@Pango.default_break].
  *
  * If @offset is not -1, it is used to apply attributes
  * from @analysis that are relevant to line breaking.
@@ -1825,11 +1839,13 @@ tailor_segment (const char      *range_start,
  *
  * Computes a `PangoLogAttr` for each character in @text.
  *
- * The @log_attrs array must have one `PangoLogAttr` for each position in @text;
- * if @text contains N characters, it has N+1 positions, including the last
- * position at the end of the text. @text should be an entire paragraph; logical
- * attributes can't be computed without context (for example you need to see
- * spaces on either side of a word to know the word is a word).
+ * The @log_attrs array must have one `PangoLogAttr` for
+ * each position in @text; if @text contains N characters,
+ * it has N+1 positions, including the last position at the
+ * end of the text. @text should be an entire paragraph;
+ * logical attributes can't be computed without context
+ * (for example you need to see spaces on either side of
+ * a word to know the word is a word).
  */
 void
 pango_get_log_attrs (const char    *text,
