@@ -153,7 +153,7 @@ pango_renderer_finalize (GObject *gobject)
  * @x: X position of left edge of baseline, in user space coordinates
  *   in Pango units.
  * @y: Y position of left edge of baseline, in user space coordinates
- *    in Pango units.
+ *   in Pango units.
  *
  * Draws @layout with the specified `PangoRenderer`.
  *
@@ -282,18 +282,22 @@ draw_strikethrough (PangoRenderer *renderer,
                     LineState     *state)
 {
   PangoRectangle *rect = &state->strikethrough_rect;
-  gboolean strikethrough = state->strikethrough;
   int num_glyphs = state->strikethrough_glyphs;
 
-  state->strikethrough = FALSE;
-
-  if (strikethrough)
+  if (state->strikethrough)
     pango_renderer_draw_rectangle (renderer,
                                    PANGO_RENDER_PART_STRIKETHROUGH,
                                    rect->x,
                                    rect->y / num_glyphs,
                                    rect->width,
                                    rect->height / num_glyphs);
+
+  state->strikethrough = FALSE;
+  state->strikethrough_glyphs = 0;
+  rect->x += rect->width;
+  rect->width = 0;
+  rect->y = 0;
+  rect->height = 0;
 }
 
 static void
@@ -336,8 +340,6 @@ handle_line_state_change (PangoRenderer  *renderer,
       rect->width = state->logical_rect_end - rect->x;
       draw_strikethrough (renderer, state);
       state->strikethrough = renderer->strikethrough;
-      rect->x = state->logical_rect_end;
-      rect->width = 0;
     }
 }
 
@@ -484,7 +486,6 @@ add_strikethrough (PangoRenderer    *renderer,
     }
   else
     {
-      draw_strikethrough (renderer, state);
       *current_rect = new_rect;
       state->strikethrough = TRUE;
       state->strikethrough_glyphs = num_glyphs;
@@ -557,7 +558,7 @@ draw_shaped_glyphs (PangoRenderer    *renderer,
  * @x: X position of left edge of baseline, in user space coordinates
  *   in Pango units.
  * @y: Y position of left edge of baseline, in user space coordinates
- *    in Pango units.
+ *   in Pango units.
  *
  * Draws @line with the specified `PangoRenderer`.
  *
@@ -750,7 +751,7 @@ pango_renderer_draw_layout_line (PangoRenderer   *renderer,
  * @x: X position of left edge of baseline, in user space coordinates
  *   in Pango units.
  * @y: Y position of left edge of baseline, in user space coordinates
- *    in Pango units.
+ *   in Pango units.
  *
  * Draws the glyphs in @glyphs with the specified `PangoRenderer`.
  *
@@ -801,12 +802,12 @@ pango_renderer_default_draw_glyphs (PangoRenderer    *renderer,
 /**
  * pango_renderer_draw_glyph_item:
  * @renderer: a `PangoRenderer`
- * @text: (allow-none): the UTF-8 text that @glyph_item refers to, or %NULL
+ * @text: (nullable): the UTF-8 text that @glyph_item refers to
  * @glyph_item: a `PangoGlyphItem`
  * @x: X position of left edge of baseline, in user space coordinates
- *   in Pango units.
+ *   in Pango units
  * @y: Y position of left edge of baseline, in user space coordinates
- *    in Pango units.
+ *   in Pango units
  *
  * Draws the glyphs in @glyph_item with the specified `PangoRenderer`,
  * embedding the text associated with the glyphs in the output if the
@@ -1206,7 +1207,7 @@ pango_renderer_draw_trapezoid (PangoRenderer   *renderer,
 /**
  * pango_renderer_draw_glyph:
  * @renderer: a `PangoRenderer`
- * @font: a #PangoFont
+ * @font: a `PangoFont`
  * @glyph: the glyph index of a single glyph
  * @x: X coordinate of left edge of baseline of glyph
  * @y: Y coordinate of left edge of baseline of glyph
@@ -1237,16 +1238,17 @@ pango_renderer_draw_glyph (PangoRenderer *renderer,
  * @renderer: a `PangoRenderer`
  *
  * Does initial setup before rendering operations on @renderer.
- * 
+ *
  * [method@Pango.Renderer.deactivate] should be called when done drawing.
  * Calls such as [method@Pango.Renderer.draw_layout] automatically
- * activate the layout before drawing on it. Calls to
- * `pango_renderer_activate()` and `pango_renderer_deactivate()`
- * can be nested and the renderer will only be initialized and
- * deinitialized once.
+ * activate the layout before drawing on it.
+ *
+ * Calls to [method@Pango.Renderer.activate] and
+ * [method@Pango.Renderer.deactivate] can be nested and the
+ * renderer will only be initialized and deinitialized once.
  *
  * Since: 1.8
- **/
+ */
 void
 pango_renderer_activate (PangoRenderer *renderer)
 {
@@ -1288,7 +1290,7 @@ pango_renderer_deactivate (PangoRenderer *renderer)
  * pango_renderer_set_color:
  * @renderer: a `PangoRenderer`
  * @part: the part to change the color of
- * @color: (allow-none): the new color or %NULL to unset the current color
+ * @color: (nullable): the new color or %NULL to unset the current color
  *
  * Sets the color for part of the rendering.
  *
@@ -1552,7 +1554,7 @@ pango_renderer_default_prepare_run (PangoRenderer  *renderer,
 /**
  * pango_renderer_set_matrix:
  * @renderer: a `PangoRenderer`
- * @matrix: (allow-none): a `PangoMatrix`, or %NULL to unset any existing matrix.
+ * @matrix: (nullable): a `PangoMatrix`, or %NULL to unset any existing matrix
  *  (No matrix set is the same as setting the identity matrix.)
  *
  * Sets the transformation matrix that will be applied when rendering.
