@@ -91,6 +91,25 @@ test_attributes_equal (void)
   pango_attribute_destroy (attr3);
 }
 
+static void
+test_attributes_register (void)
+{
+  PangoAttrType type;
+  GEnumClass *class;
+
+  type = pango_attr_type_register ("my-attribute");
+  g_assert_cmpstr (pango_attr_type_get_name (type), ==, "my-attribute");
+
+  class = g_type_class_ref (PANGO_TYPE_ATTR_TYPE);
+  for (int i = 0; i < class->n_values; i++)
+    {
+      g_assert_cmpint (type, !=, class->values[i].value);
+      g_assert_null (pango_attr_type_get_name (class->values[i].value));
+    }
+
+  g_type_class_unref (class);
+}
+
 static void
 assert_attributes (GSList     *attrs,
                    const char *expected)
@@ -101,7 +120,7 @@ assert_attributes (GSList     *attrs,
   print_attributes (attrs, s);
   if (strcmp (s->str, expected) != 0)
     {
-      g_print ("-----\nattribute list mismatch\nexpected:\n%s-----\nreceived:\n%s-----\n",
+      g_print ("-----\nattribute list mismatch\nexpected:\n%s\n-----\nreceived:\n%s\n-----\n",
                expected, s->str);
       g_assert_not_reached ();
     }
@@ -112,11 +131,21 @@ static void
 assert_attr_list (PangoAttrList *list,
                   const char    *expected)
 {
-  GSList *attrs;
+  PangoAttrList *list2;
 
-  attrs = pango_attr_list_get_attributes (list);
-  assert_attributes (attrs, expected);
-  g_slist_free_full (attrs, (GDestroyNotify)pango_attribute_destroy);
+  list2 = attributes_from_string (expected);
+  if (!pango_attr_list_equal (list, list2))
+    {
+      GString *s = g_string_new ("");
+
+      print_attr_list (list, s);
+      g_print ("-----\nattribute list mismatch\nexpected:\n%s-----\nreceived:\n%s-----\n",
+               expected, s->str);
+      g_string_free (s, TRUE);
+      g_assert_not_reached ();
+    }
+
+  pango_attr_list_unref (list2);
 }
 
 static void
@@ -133,7 +162,7 @@ assert_attr_iterator (PangoAttrIterator *iter,
 static void
 test_list (void)
 {
-  PangoAttrList *list;
+  PangoAttrList *list, *list2;
   PangoAttribute *attr;
 
   list = pango_attr_list_new ();
@@ -145,6 +174,12 @@ test_list (void)
   attr = pango_attr_size_new (30);
   pango_attr_list_insert (list, attr);
 
+  list2 = attributes_from_string ("[0,-1]size=10\n"
+                                  "[0,-1]size=20\n"
+                                  "[0,-1]size=30\n");
+  g_assert_true (pango_attr_list_equal (list, list2));
+  pango_attr_list_unref (list2);
+
   assert_attr_list (list, "[0,-1]size=10\n"
                           "[0,-1]size=20\n"
                           "[0,-1]size=30\n");
@@ -153,17 +188,13 @@ test_list (void)
   list = pango_attr_list_new ();
 
   /* test that insertion respects start_index */
-  attr = pango_attr_size_new (10);
+  attr = attribute_from_string ("[0,-1]size=10");
   pango_attr_list_insert (list, attr);
-  attr = pango_attr_size_new (20);
-  attr->start_index = 10;
-  attr->end_index = 20;
+  attr = attribute_from_string ("[10,20]size=20");
   pango_attr_list_insert (list, attr);
-  attr = pango_attr_size_new (30);
+  attr = attribute_from_string ("[0,-1]size=30");
   pango_attr_list_insert (list, attr);
-  attr = pango_attr_size_new (40);
-  attr->start_index = 10;
-  attr->end_index = 40;
+  attr = attribute_from_string ("[10,40]size=40");
   pango_attr_list_insert_before (list, attr);
 
   assert_attr_list (list, "[0,-1]size=10\n"
@@ -179,29 +210,20 @@ test_list_change (void)
   PangoAttrList *list;
   PangoAttribute *attr;
 
-  list = pango_attr_list_new ();
+  list = attributes_from_string ("[0,10]size=10\n"
+                                 "[0,30]weight=700\n"
+                                 "[20,30]size=20\n");
 
-  attr = pango_attr_size_new (10);
-  attr->start_index = 0;
-  attr->end_index = 10;
-  pango_attr_list_insert (list, attr);
-  attr = pango_attr_size_new (20);
-  attr->start_index = 20;
-  attr->end_index = 30;
-  pango_attr_list_insert (list, attr);
-  attr = pango_attr_weight_new (PANGO_WEIGHT_BOLD);
-  attr->start_index = 0;
-  attr->end_index = 30;
-  pango_attr_list_insert (list, attr);
+  /* no-op */
+  attr = attribute_from_string ("[10,10]variant=small-caps");
+  pango_attr_list_change (list, attr);
 
   assert_attr_list (list, "[0,10]size=10\n"
                           "[0,30]weight=700\n"
                           "[20,30]size=20\n");
 
   /* simple insertion with pango_attr_list_change */
-  attr = pango_attr_variant_new (PANGO_VARIANT_SMALL_CAPS);
-  attr->start_index = 10;
-  attr->end_index = 20;
+  attr = attribute_from_string ("[10,20]variant=small-caps");
   pango_attr_list_change (list, attr);
 
   assert_attr_list (list, "[0,10]size=10\n"
@@ -210,9 +232,7 @@ test_list_change (void)
                           "[20,30]size=20\n");
 
   /* insertion with splitting */
-  attr = pango_attr_weight_new (PANGO_WEIGHT_LIGHT);
-  attr->start_index = 15;
-  attr->end_index = 20;
+  attr = attribute_from_string ("[15,20]weight=light");
   pango_attr_list_change (list, attr);
 
   assert_attr_list (list, "[0,10]size=10\n"
@@ -223,9 +243,7 @@ test_list_change (void)
                           "[20,30]weight=700\n");
 
   /* insertion with joining */
-  attr = pango_attr_size_new (20);
-  attr->start_index = 5;
-  attr->end_index = 20;
+  attr = attribute_from_string ("[5,20]size=20");
   pango_attr_list_change (list, attr);
 
   assert_attr_list (list, "[0,5]size=10\n"
@@ -238,31 +256,270 @@ test_list_change (void)
   pango_attr_list_unref (list);
 }
 
+/* See https://gitlab.gnome.org/GNOME/pango/-/issues/564 */
+static void
+test_list_change2 (void)
+{
+  PangoAttrList *list;
+  PangoAttribute *attr;
+
+  list = attributes_from_string ("[0,3]style=italic\n"
+                                 "[3,4]style=normal\n"
+                                 "[4,18]style=italic\n");
+
+  /* insertion with joining */
+  attr = attribute_from_string ("[0,18]style=normal");
+  pango_attr_list_change (list, attr);
+
+  assert_attr_list (list, "[0,18]style=normal\n");
+
+  pango_attr_list_unref (list);
+}
+
+static void
+test_list_change3 (void)
+{
+  PangoAttrList *list;
+  PangoAttribute *attr;
+
+  list = attributes_from_string ("[0,3]style=italic\n"
+                                 "[3,4]style=normal\n"
+                                 "[4,18]style=italic\n");
+
+  /* insertion with joining */
+  attr = attribute_from_string ("[1,1]style=normal");
+  pango_attr_list_change (list, attr);
+
+  assert_attr_list (list, "[0,3]style=italic\n"
+                          "[3,4]style=normal\n"
+                          "[4,18]style=italic\n");
+
+  pango_attr_list_unref (list);
+}
+
+static void
+test_list_change4 (void)
+{
+  PangoAttrList *list;
+  PangoAttribute *attr;
+
+  list = pango_attr_list_new ();
+
+  /* insertion with joining */
+  attr = attribute_from_string ("[0,10]style=normal");
+  pango_attr_list_change (list, attr);
+
+  assert_attr_list (list, "[0,10]style=normal");
+
+  pango_attr_list_unref (list);
+}
+
+static void
+test_list_change5 (void)
+{
+  PangoAttrList *list;
+  PangoAttribute *attr;
+
+  list = attributes_from_string ("[0,3]weight=800\n"
+                                 "[2,4]style=normal\n"
+                                 "[10,20]style=normal\n"
+                                 "[15,18]fallback=false\n");
+
+  /* insertion with joining */
+  attr = attribute_from_string ("[5,15]style=italic");
+  pango_attr_list_change (list, attr);
+
+  assert_attr_list (list, "[0,3]weight=800\n"
+                          "[2,4]style=normal\n"
+                          "[5,15]style=italic\n"
+                          "[15,20]style=normal\n"
+                          "[15,18]fallback=false");
+
+  pango_attr_list_unref (list);
+}
+
+static void
+test_list_change6 (void)
+{
+  PangoAttrList *list;
+  PangoAttribute *attr;
+
+  list = attributes_from_string ("[0,3]weight=800\n"
+                                 "[2,4]style=normal\n"
+                                 "[10,20]style=normal\n"
+                                 "[15,18]fallback=false\n");
+
+  /* insertion with joining */
+  attr = attribute_from_string ("[3,10]style=normal");
+  pango_attr_list_change (list, attr);
+
+  assert_attr_list (list, "[0,3]weight=800\n"
+                          "[2,20]style=normal\n"
+                          "[15,18]fallback=false");
+
+  pango_attr_list_unref (list);
+}
+
+static void
+test_list_change7 (void)
+{
+  PangoAttrList *list;
+  PangoAttribute *attr;
+
+  list = attributes_from_string ("[0,3]weight=800\n"
+                                 "[2,4]style=normal\n"
+                                 "[10,20]style=normal\n"
+                                 "[15,18]fallback=false\n");
+
+  /* insertion with joining */
+  attr = attribute_from_string ("[3,4]style=normal");
+  pango_attr_list_change (list, attr);
+
+  assert_attr_list (list, "[0,3]weight=800\n"
+                          "[2,4]style=normal\n"
+                          "[10,20]style=normal\n"
+                          "[15,18]fallback=false");
+
+  pango_attr_list_unref (list);
+}
+
+static void
+test_list_change8 (void)
+{
+  PangoAttrList *list;
+  PangoAttribute *attr;
+
+  list = attributes_from_string ("[0,3]weight=800\n"
+                                 "[2,4]style=normal\n"
+                                 "[10,20]style=normal\n"
+                                 "[15,18]fallback=false\n");
+
+  /* insertion with joining */
+  attr = attribute_from_string ("[3,11]style=italic");
+  pango_attr_list_change (list, attr);
+
+  assert_attr_list (list, "[0,3]weight=800\n"
+                          "[2,3]style=normal\n"
+                          "[3,11]style=italic\n"
+                          "[11,20]style=normal\n"
+                          "[15,18]fallback=false");
+
+  pango_attr_list_unref (list);
+}
+
+static void
+test_list_change9 (void)
+{
+  PangoAttrList *list;
+  PangoAttribute *attr;
+
+  list = attributes_from_string ("[0,3]weight=800\n"
+                                 "[2,4]style=normal\n"
+                                 "[10,20]style=normal\n"
+                                 "[15,18]fallback=false\n");
+
+  /* insertion with joining */
+  attr = attribute_from_string ("[2,3]style=italic");
+  pango_attr_list_change (list, attr);
+
+  assert_attr_list (list, "[0,3]weight=800\n"
+                          "[2,3]style=italic\n"
+                          "[3,4]style=normal\n"
+                          "[10,20]style=normal\n"
+                          "[15,18]fallback=false");
+
+  pango_attr_list_unref (list);
+}
+
+static void
+test_list_change10 (void)
+{
+  PangoAttrList *list;
+  PangoAttribute *attr;
+
+  list = attributes_from_string ("[0,3]weight=800\n"
+                                 "[2,4]style=normal\n"
+                                 "[10,20]style=normal\n"
+                                 "[15,18]fallback=false\n");
+
+  /* insertion with joining */
+  attr = attribute_from_string ("[3,4]style=italic");
+  pango_attr_list_change (list, attr);
+
+  assert_attr_list (list, "[0,3]weight=800\n"
+                          "[2,3]style=normal\n"
+                          "[3,4]style=italic\n"
+                          "[10,20]style=normal\n"
+                          "[15,18]fallback=false");
+
+  pango_attr_list_unref (list);
+}
+
+static void
+test_list_change11 (void)
+{
+  PangoAttrList *list;
+  PangoAttribute *attr;
+
+  list = attributes_from_string ("[0,3]weight=800\n"
+                                 "[2,4]style=normal\n"
+                                 "[3,5]fallback=false\n"
+                                 "[10,20]style=italic\n"
+                                 "[15,18]fallback=false\n"
+                                 "[22,30]style=italic\n");
+
+  /* insertion with joining */
+  attr = attribute_from_string ("[3,22]style=italic");
+  pango_attr_list_change (list, attr);
+
+  assert_attr_list (list, "[0,3]weight=800\n"
+                          "[2,3]style=normal\n"
+                          "[3,30]style=italic\n"
+                          "[3,5]fallback=false\n"
+                          "[15,18]fallback=false\n");
+
+  pango_attr_list_unref (list);
+}
+
+static void
+test_list_change12 (void)
+{
+  PangoAttrList *list;
+  PangoAttribute *attr;
+
+  list = attributes_from_string ("[0,3]weight=800\n"
+                                 "[2,4]style=normal\n"
+                                 "[3,5]fallback=false\n"
+                                 "[10,20]style=normal\n"
+                                 "[15,18]fallback=false\n"
+                                 "[20,30]style=oblique\n"
+                                 "[21,22]fallback=false");
+
+  /* insertion with joining */
+  attr = attribute_from_string ("[3,22]style=italic");
+  pango_attr_list_change (list, attr);
+
+  assert_attr_list (list, "[0,3]weight=800\n"
+                          "[2,3]style=normal\n"
+                          "[3,22]style=italic\n"
+                          "[3,5]fallback=false\n"
+                          "[15,18]fallback=false\n"
+                          "[21,22]fallback=false\n"
+                          "[22,30]style=oblique");
+
+  pango_attr_list_unref (list);
+}
+
 static void
 test_list_splice (void)
 {
   PangoAttrList *base;
   PangoAttrList *list;
   PangoAttrList *other;
-  PangoAttribute *attr;
 
-  base = pango_attr_list_new ();
-  attr = pango_attr_size_new (10);
-  attr->start_index = 0;
-  attr->end_index = -1;
-  pango_attr_list_insert (base, attr);
-  attr = pango_attr_weight_new (PANGO_WEIGHT_BOLD);
-  attr->start_index = 10;
-  attr->end_index = 15;
-  pango_attr_list_insert (base, attr);
-  attr = pango_attr_variant_new (PANGO_VARIANT_SMALL_CAPS);
-  attr->start_index = 20;
-  attr->end_index = 30;
-  pango_attr_list_insert (base, attr);
-
-  assert_attr_list (base, "[0,-1]size=10\n"
-                          "[10,15]weight=700\n"
-                          "[20,30]variant=1\n");
+  base = attributes_from_string ("[0,-1]size=10\n"
+                                 "[10,15]weight=700\n"
+                                 "[20,30]variant=1\n");
 
   /* splice in an empty list */
   list = pango_attr_list_copy (base);
@@ -278,15 +535,9 @@ test_list_splice (void)
 
   /* splice in some attributes */
   list = pango_attr_list_copy (base);
-  other = pango_attr_list_new ();
-  attr = pango_attr_size_new (20);
-  attr->start_index = 0;
-  attr->end_index = 3;
-  pango_attr_list_insert (other, attr);
-  attr = pango_attr_stretch_new (PANGO_STRETCH_CONDENSED);
-  attr->start_index = 2;
-  attr->end_index = 4;
-  pango_attr_list_insert (other, attr);
+
+  other = attributes_from_string ("[0,3]size=20\n"
+                                  "[2,4]stretch=2\n");
 
   pango_attr_list_splice (list, other, 11, 5);
 
@@ -318,9 +569,7 @@ test_list_splice2 (void)
 
   g_assert_null (pango_attr_list_get_attributes (list));
 
-  attr = pango_attr_size_new (10);
-  attr->start_index = 0;
-  attr->end_index = -1;
+  attr = attribute_from_string ("[0,-1]size=10");
   pango_attr_list_insert (other, attr);
 
   pango_attr_list_splice (list, other, 11, 5);
@@ -344,23 +593,9 @@ test_list_splice3 (void)
 {
   PangoAttrList *list;
   PangoAttrList *other;
-  PangoAttribute *attr;
 
-  list = pango_attr_list_new ();
-  other = pango_attr_list_new ();
-
-  attr = pango_attr_variant_new (PANGO_VARIANT_SMALL_CAPS);
-  attr->start_index = 10;
-  attr->end_index = 30;
-  pango_attr_list_insert (list, attr);
-
-  attr = pango_attr_weight_new (PANGO_WEIGHT_BOLD);
-  attr->start_index = PANGO_ATTR_INDEX_FROM_TEXT_BEGINNING;
-  attr->end_index = PANGO_ATTR_INDEX_TO_TEXT_END;
-  pango_attr_list_insert (other, attr);
-
-  assert_attr_list (list, "[10,30]variant=1\n");
-  assert_attr_list (other, "[0,-1]weight=700\n");
+  list = attributes_from_string ("[10,30]variant=1\n");
+  other = attributes_from_string ("[0,-1]weight=700\n");
 
   pango_attr_list_splice (list, other, 20, 5);
 
@@ -391,22 +626,10 @@ test_list_filter (void)
 {
   PangoAttrList *list;
   PangoAttrList *out;
-  PangoAttribute *attr;
 
-  list = pango_attr_list_new ();
-  attr = pango_attr_size_new (10);
-  pango_attr_list_insert (list, attr);
-  attr = pango_attr_stretch_new (PANGO_STRETCH_CONDENSED);
-  attr->start_index = 10;
-  attr->end_index = 20;
-  pango_attr_list_insert (list, attr);
-  attr = pango_attr_weight_new (PANGO_WEIGHT_BOLD);
-  attr->start_index = 20;
-  pango_attr_list_insert (list, attr);
-
-  assert_attr_list (list, "[0,-1]size=10\n"
-                          "[10,20]stretch=2\n"
-                          "[20,-1]weight=700\n");
+  list = attributes_from_string ("[0,-1]size=10\n"
+                                 "[10,20]stretch=2\n"
+                                 "[20,-1]weight=700\n");
 
   out = pango_attr_list_filter (list, never_true, NULL);
   g_assert_null (out);
@@ -441,14 +664,11 @@ test_iter (void)
   pango_attr_list_unref (list);
 
   list = pango_attr_list_new ();
-  attr = pango_attr_size_new (10);
+  attr = attribute_from_string ("[0,-1]size=10");
   pango_attr_list_insert (list, attr);
-  attr = pango_attr_stretch_new (PANGO_STRETCH_CONDENSED);
-  attr->start_index = 10;
-  attr->end_index = 30;
+  attr = attribute_from_string ("[10,30]stretch=condensed");
   pango_attr_list_insert (list, attr);
-  attr = pango_attr_weight_new (PANGO_WEIGHT_BOLD);
-  attr->start_index = 20;
+  attr = attribute_from_string ("[20,-1]weight=bold");
   pango_attr_list_insert (list, attr);
 
   iter = pango_attr_list_get_iterator (list);
@@ -494,12 +714,9 @@ test_iter_get (void)
   list = pango_attr_list_new ();
   attr = pango_attr_size_new (10);
   pango_attr_list_insert (list, attr);
-  attr = pango_attr_stretch_new (PANGO_STRETCH_CONDENSED);
-  attr->start_index = 10;
-  attr->end_index = 30;
+  attr = attribute_from_string ("[10,30]stretch=condensed");
   pango_attr_list_insert (list, attr);
-  attr = pango_attr_weight_new (PANGO_WEIGHT_BOLD);
-  attr->start_index = 20;
+  attr = attribute_from_string ("[20,-1]weight=bold");
   pango_attr_list_insert (list, attr);
 
   iter = pango_attr_list_get_iterator (list);
@@ -535,21 +752,17 @@ test_iter_get_font (void)
   list = pango_attr_list_new ();
   attr = pango_attr_size_new (10 * PANGO_SCALE);
   pango_attr_list_insert (list, attr);
-  attr = pango_attr_family_new ("Times");
+  attr = attribute_from_string ("[0,-1]family=Times");
   pango_attr_list_insert (list, attr);
-  attr = pango_attr_stretch_new (PANGO_STRETCH_CONDENSED);
-  attr->start_index = 10;
-  attr->end_index = 30;
+  attr = attribute_from_string ("[10,30]stretch=condensed");
   pango_attr_list_insert (list, attr);
-  attr = pango_attr_language_new (pango_language_from_string ("ja-JP"));
-  attr->start_index = 10;
-  attr->end_index = 20;
+  attr = attribute_from_string ("[10,20]absolute-size=10240");
   pango_attr_list_insert (list, attr);
-  attr = pango_attr_rise_new (100);
-  attr->start_index = 20;
+  attr = attribute_from_string ("[10,20]language=ja-JP");
   pango_attr_list_insert (list, attr);
-  attr = pango_attr_fallback_new (FALSE);
-  attr->start_index = 20;
+  attr = attribute_from_string ("[20,-1]rise=100");
+  pango_attr_list_insert (list, attr);
+  attr = attribute_from_string ("[20,-1]fallback=0");
   pango_attr_list_insert (list, attr);
 
   iter = pango_attr_list_get_iterator (list);
@@ -565,7 +778,7 @@ test_iter_get_font (void)
   pango_attr_iterator_next (iter);
   desc = pango_font_description_new ();
   pango_attr_iterator_get_font (iter, desc, &lang, &attrs);
-  desc2 = pango_font_description_from_string ("Times Condensed 10");
+  desc2 = pango_font_description_from_string ("Times Condensed 10px");
   g_assert_true (pango_font_description_equal (desc, desc2));
   g_assert_nonnull (lang);
   g_assert_cmpstr (pango_language_to_string (lang), ==, "ja-jp");
@@ -600,21 +813,15 @@ test_iter_get_attrs (void)
   list = pango_attr_list_new ();
   attr = pango_attr_size_new (10 * PANGO_SCALE);
   pango_attr_list_insert (list, attr);
-  attr = pango_attr_family_new ("Times");
+  attr = attribute_from_string ("[0,-1]family=Times");
   pango_attr_list_insert (list, attr);
-  attr = pango_attr_stretch_new (PANGO_STRETCH_CONDENSED);
-  attr->start_index = 10;
-  attr->end_index = 30;
+  attr = attribute_from_string ("[10,30]stretch=condensed");
   pango_attr_list_insert (list, attr);
-  attr = pango_attr_language_new (pango_language_from_string ("ja-JP"));
-  attr->start_index = 10;
-  attr->end_index = 20;
+  attr = attribute_from_string ("[10,20]language=ja-JP");
   pango_attr_list_insert (list, attr);
-  attr = pango_attr_rise_new (100);
-  attr->start_index = 20;
+  attr = attribute_from_string ("[20,-1]rise=100");
   pango_attr_list_insert (list, attr);
-  attr = pango_attr_fallback_new (FALSE);
-  attr->start_index = 20;
+  attr = attribute_from_string ("[20,-1]fallback=0");
   pango_attr_list_insert (list, attr);
 
   iter = pango_attr_list_get_iterator (list);
@@ -651,35 +858,12 @@ static void
 test_list_update (void)
 {
   PangoAttrList *list;
-  PangoAttribute *attr;
 
-  list = pango_attr_list_new ();
-  attr = pango_attr_size_new (10 * PANGO_SCALE);
-  attr->start_index = 10;
-  attr->end_index = 11;
-  pango_attr_list_insert (list, attr);
-  attr = pango_attr_rise_new (100);
-  attr->start_index = 0;
-  attr->end_index = 200;
-  pango_attr_list_insert (list, attr);
-  attr = pango_attr_family_new ("Times");
-  attr->start_index = 5;
-  attr->end_index = 15;
-  pango_attr_list_insert (list, attr);
-  attr = pango_attr_fallback_new (FALSE);
-  attr->start_index = 11;
-  attr->end_index = 100;
-  pango_attr_list_insert (list, attr);
-  attr = pango_attr_stretch_new (PANGO_STRETCH_CONDENSED);
-  attr->start_index = 30;
-  attr->end_index = 60;
-  pango_attr_list_insert (list, attr);
-
-  assert_attr_list (list, "[0,200]rise=100\n"
-                          "[5,15]family=Times\n"
-                          "[10,11]size=10240\n"
-                          "[11,100]fallback=0\n"
-                          "[30,60]stretch=2\n");
+  list = attributes_from_string ("[0,200]rise=100\n"
+                                 "[5,15]family=Times\n"
+                                 "[10,11]size=10240\n"
+                                 "[11,100]fallback=0\n"
+                                 "[30,60]stretch=2\n");
 
   pango_attr_list_update (list, 8, 10, 20);
 
@@ -705,6 +889,20 @@ test_list_update2 (void)
   pango_attr_list_unref (list);
 }
 
+/* test overflow */
+static void
+test_list_update3 (void)
+{
+  PangoAttrList *list;
+
+  list = attributes_from_string ("[5,4294967285]family=Times\n");
+
+  pango_attr_list_update (list, 8, 10, 30);
+
+  assert_attr_list (list, "[5,-1]family=Times\n");
+
+  pango_attr_list_unref (list);
+}
 static void
 test_list_equal (void)
 {
@@ -720,18 +918,14 @@ test_list_equal (void)
   g_assert_true (pango_attr_list_equal (list1, list1));
   g_assert_true (pango_attr_list_equal (list1, list2));
 
-  attr = pango_attr_size_new (10 * PANGO_SCALE);
-  attr->start_index = 0;
-  attr->end_index = 7;
+  attr = attribute_from_string ("[0,7]size=10240");
   pango_attr_list_insert (list1, pango_attribute_copy (attr));
   pango_attr_list_insert (list2, pango_attribute_copy (attr));
   pango_attribute_destroy (attr);
 
   g_assert_true (pango_attr_list_equal (list1, list2));
 
-  attr = pango_attr_stretch_new (PANGO_STRETCH_CONDENSED);
-  attr->start_index = 0;
-  attr->end_index = 1;
+  attr = attribute_from_string ("[0,1]stretch=condensed");
   pango_attr_list_insert (list1, pango_attribute_copy (attr));
   g_assert_true (!pango_attr_list_equal (list1, list2));
 
@@ -739,10 +933,8 @@ test_list_equal (void)
   g_assert_true (pango_attr_list_equal (list1, list2));
   pango_attribute_destroy (attr);
 
-  attr = pango_attr_size_new (30 * PANGO_SCALE);
   /* Same range as the first attribute */
-  attr->start_index = 0;
-  attr->end_index = 7;
+  attr = attribute_from_string ("[0,7]size=30720");
   pango_attr_list_insert (list2, pango_attribute_copy (attr));
   g_assert_true (!pango_attr_list_equal (list1, list2));
   pango_attr_list_insert (list1, pango_attribute_copy (attr));
@@ -787,37 +979,13 @@ test_insert (void)
   PangoAttrList *list;
   PangoAttribute *attr;
 
-  list = pango_attr_list_new ();
-  attr = pango_attr_size_new (10 * PANGO_SCALE);
-  attr->start_index = 10;
-  attr->end_index = 11;
-  pango_attr_list_insert (list, attr);
-  attr = pango_attr_rise_new (100);
-  attr->start_index = 0;
-  attr->end_index = 200;
-  pango_attr_list_insert (list, attr);
-  attr = pango_attr_family_new ("Times");
-  attr->start_index = 5;
-  attr->end_index = 15;
-  pango_attr_list_insert (list, attr);
-  attr = pango_attr_fallback_new (FALSE);
-  attr->start_index = 11;
-  attr->end_index = 100;
-  pango_attr_list_insert (list, attr);
-  attr = pango_attr_stretch_new (PANGO_STRETCH_CONDENSED);
-  attr->start_index = 30;
-  attr->end_index = 60;
-  pango_attr_list_insert (list, attr);
+  list = attributes_from_string ("[0,200]rise=100\n"
+                                 "[5,15]family=Times\n"
+                                 "[10,11]size=10240\n"
+                                 "[11,100]fallback=0\n"
+                                 "[30,60]stretch=2\n");
 
-  assert_attr_list (list, "[0,200]rise=100\n"
-                          "[5,15]family=Times\n"
-                          "[10,11]size=10240\n"
-                          "[11,100]fallback=0\n"
-                          "[30,60]stretch=2\n");
-
-  attr = pango_attr_family_new ("Times");
-  attr->start_index = 10;
-  attr->end_index = 25;
+  attr = attribute_from_string ("[10,25]family=Times");
   pango_attr_list_change (list, attr);
 
   assert_attr_list (list, "[0,200]rise=100\n"
@@ -826,9 +994,7 @@ test_insert (void)
                           "[11,100]fallback=0\n"
                           "[30,60]stretch=2\n");
 
-  attr = pango_attr_family_new ("Futura");
-  attr->start_index = 11;
-  attr->end_index = 25;
+  attr = attribute_from_string ("[11,25]family=Futura");
   pango_attr_list_insert (list, attr);
 
   assert_attr_list (list, "[0,200]rise=100\n"
@@ -841,6 +1007,33 @@ test_insert (void)
   pango_attr_list_unref (list);
 }
 
+static void
+test_insert2 (void)
+{
+  PangoAttrList *list;
+  PangoAttribute *attr;
+
+  list = attributes_from_string ("[0,200]rise=100\n"
+                                 "[5,15]family=Times\n"
+                                 "[10,11]size=10240\n"
+                                 "[11,100]fallback=0\n"
+                                 "[20,30]family=Times\n"
+                                 "[30,40]family=Futura\n"
+                                 "[30,60]stretch=2\n");
+
+  attr = attribute_from_string ("[10,35]family=Times");
+  pango_attr_list_change (list, attr);
+
+  assert_attr_list (list, "[0,200]rise=100\n"
+                          "[5,35]family=Times\n"
+                          "[10,11]size=10240\n"
+                          "[11,100]fallback=0\n"
+                          "[35,40]family=Futura\n"
+                          "[30,60]stretch=2\n");
+
+  pango_attr_list_unref (list);
+}
+
 static gboolean
 attr_list_merge_filter (PangoAttribute *attribute,
                         gpointer        list)
@@ -855,53 +1048,16 @@ test_merge (void)
 {
   PangoAttrList *list;
   PangoAttrList *list2;
-  PangoAttribute *attr;
 
-  list = pango_attr_list_new ();
-  attr = pango_attr_size_new (10 * PANGO_SCALE);
-  attr->start_index = 10;
-  attr->end_index = 11;
-  pango_attr_list_insert (list, attr);
-  attr = pango_attr_rise_new (100);
-  attr->start_index = 0;
-  attr->end_index = 200;
-  pango_attr_list_insert (list, attr);
-  attr = pango_attr_family_new ("Times");
-  attr->start_index = 5;
-  attr->end_index = 15;
-  pango_attr_list_insert (list, attr);
-  attr = pango_attr_fallback_new (FALSE);
-  attr->start_index = 11;
-  attr->end_index = 100;
-  pango_attr_list_insert (list, attr);
-  attr = pango_attr_stretch_new (PANGO_STRETCH_CONDENSED);
-  attr->start_index = 30;
-  attr->end_index = 60;
-  pango_attr_list_insert (list, attr);
+  list = attributes_from_string ("[0,200]rise=100\n"
+                                 "[5,15]family=Times\n"
+                                 "[10,11]size=10240\n"
+                                 "[11,100]fallback=0\n"
+                                 "[30,60]stretch=2\n");
 
-  assert_attr_list (list, "[0,200]rise=100\n"
-                          "[5,15]family=Times\n"
-                          "[10,11]size=10240\n"
-                          "[11,100]fallback=0\n"
-                          "[30,60]stretch=2\n");
-
-  list2 = pango_attr_list_new ();
-  attr = pango_attr_size_new (10 * PANGO_SCALE);
-  attr->start_index = 11;
-  attr->end_index = 13;
-  pango_attr_list_insert (list2, attr);
-  attr = pango_attr_size_new (11 * PANGO_SCALE);
-  attr->start_index = 13;
-  attr->end_index = 15;
-  pango_attr_list_insert (list2, attr);
-  attr = pango_attr_size_new (12 * PANGO_SCALE);
-  attr->start_index = 40;
-  attr->end_index = 50;
-  pango_attr_list_insert (list2, attr);
-
-  assert_attr_list (list2, "[11,13]size=10240\n"
-                           "[13,15]size=11264\n"
-                           "[40,50]size=12288\n");
+  list2 = attributes_from_string ("[11,13]size=10240\n"
+                                  "[13,15]size=11264\n"
+                                  "[40,50]size=12288\n");
 
   pango_attr_list_filter (list2, attr_list_merge_filter, list);
 
@@ -926,23 +1082,10 @@ test_merge2 (void)
   PangoAttrList *list;
   PangoAttribute *attr;
 
-  list = pango_attr_list_new ();
-  attr = pango_attr_underline_new (PANGO_UNDERLINE_SINGLE);
-  attr->start_index = 0;
-  attr->end_index = 10;
-  pango_attr_list_insert (list, attr);
-  attr = pango_attr_foreground_new (0, 0, 0xffff);
-  attr->start_index = 0;
-  attr->end_index = 10;
-  pango_attr_list_insert (list, attr);
-
-  assert_attr_list (list, "[0,10]underline=1\n"
-                          "[0,10]foreground=#00000000ffff\n");
-
-  attr = pango_attr_foreground_new (0xffff, 0, 0);
-  attr->start_index = 2;
-  attr->end_index = 3;
+  list = attributes_from_string ("[0,10]underline=1\n"
+                                 "[0,10]foreground=#00000000ffff\n");
 
+  attr = attribute_from_string ("[2,3]foreground=#ffff00000000");
   pango_attr_list_change (list, attr);
 
   assert_attr_list (list, "[0,10]underline=1\n"
@@ -950,10 +1093,7 @@ test_merge2 (void)
                           "[2,3]foreground=#ffff00000000\n"
                           "[3,10]foreground=#00000000ffff\n");
 
-  attr = pango_attr_foreground_new (0, 0xffff, 0);
-  attr->start_index = 3;
-  attr->end_index = 4;
-
+  attr = attribute_from_string ("[3,4]foreground=#0000ffff0000");
   pango_attr_list_change (list, attr);
 
   assert_attr_list (list, "[0,10]underline=1\n"
@@ -962,10 +1102,7 @@ test_merge2 (void)
                           "[3,4]foreground=#0000ffff0000\n"
                           "[4,10]foreground=#00000000ffff\n");
 
-  attr = pango_attr_foreground_new (0, 0, 0xffff);
-  attr->start_index = 4;
-  attr->end_index = 5;
-
+  attr = attribute_from_string ("[4,5]foreground=#00000000ffff");
   pango_attr_list_change (list, attr);
 
   assert_attr_list (list, "[0,10]underline=1\n"
@@ -1072,16 +1209,30 @@ main (int argc, char *argv[])
 
   g_test_add_func ("/attributes/basic", test_attributes_basic);
   g_test_add_func ("/attributes/equal", test_attributes_equal);
+  g_test_add_func ("/attributes/register", test_attributes_register);
   g_test_add_func ("/attributes/list/basic", test_list);
   g_test_add_func ("/attributes/list/change", test_list_change);
+  g_test_add_func ("/attributes/list/change2", test_list_change2);
+  g_test_add_func ("/attributes/list/change3", test_list_change3);
+  g_test_add_func ("/attributes/list/change4", test_list_change4);
+  g_test_add_func ("/attributes/list/change5", test_list_change5);
+  g_test_add_func ("/attributes/list/change6", test_list_change6);
+  g_test_add_func ("/attributes/list/change7", test_list_change7);
+  g_test_add_func ("/attributes/list/change8", test_list_change8);
+  g_test_add_func ("/attributes/list/change9", test_list_change9);
+  g_test_add_func ("/attributes/list/change10", test_list_change10);
+  g_test_add_func ("/attributes/list/change11", test_list_change11);
+  g_test_add_func ("/attributes/list/change12", test_list_change12);
   g_test_add_func ("/attributes/list/splice", test_list_splice);
   g_test_add_func ("/attributes/list/splice2", test_list_splice2);
   g_test_add_func ("/attributes/list/splice3", test_list_splice3);
   g_test_add_func ("/attributes/list/filter", test_list_filter);
   g_test_add_func ("/attributes/list/update", test_list_update);
   g_test_add_func ("/attributes/list/update2", test_list_update2);
+  g_test_add_func ("/attributes/list/update3", test_list_update3);
   g_test_add_func ("/attributes/list/equal", test_list_equal);
   g_test_add_func ("/attributes/list/insert", test_insert);
+  g_test_add_func ("/attributes/list/insert2", test_insert2);
   g_test_add_func ("/attributes/list/merge", test_merge);
   g_test_add_func ("/attributes/list/merge2", test_merge2);
   g_test_add_func ("/attributes/iter/basic", test_iter);
